"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const sfpowerkit_1 = require("../../../sfpowerkit");
const xml2js = require("xml2js");
const rimraf = require("rimraf");
const util = require("util");
const zipDirectory_1 = require("../../../utils/zipDirectory");
const checkDeploymentStatus_1 = require("../../..//utils/checkDeploymentStatus");
const core_1 = require("@salesforce/core");
const fs = require("fs-extra");
const kit_1 = require("@salesforce/kit");
const path = require("path");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "org_destruct");
class Destruct extends command_1.SfdxCommand {
    async run() {
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        await this.org.refreshAuth();
        //Connect to the org
        const conn = this.org.getConnection();
        const apiversion = await conn.retrieveMaxApiVersion();
        this.flags.apiversion =
            this.flags.apiversion || (await conn.retrieveMaxApiVersion());
        const existingManifestPath = this.flags.manifest.valueOf();
        let workingDirectory = this.generateCacheDirectory();
        try {
            await this.copyAndValidateDestructiveManifest(existingManifestPath, workingDirectory);
            this.generateEmptyPackageXml(workingDirectory, apiversion);
            let zipFile = await this.generateDeploymentZipFile(workingDirectory);
            await this.deployDestructiveManifest(zipFile, conn);
        }
        catch (e) {
            throw new core_1.SfdxError(e.message);
        }
        rimraf.sync(workingDirectory);
        return 0;
    }
    generateCacheDirectory() {
        //Setup working directory
        let cacheDirectory = fileutils_1.default.getGlobalCacheDir();
        let destructCacheDirectory = path.join(cacheDirectory, "destruct");
        //Clean existing directory
        rimraf.sync(destructCacheDirectory);
        fs.mkdirSync(destructCacheDirectory);
        return destructCacheDirectory;
    }
    async copyAndValidateDestructiveManifest(existingManifestPath, workingDirectory) {
        let destructiveManifestFile = path.join(workingDirectory, "destructiveChanges.xml");
        //Copy Destructive Manifest File to  Temporary Directory
        fs.copyFileSync(existingManifestPath, destructiveManifestFile);
        //Validate the destructive file for syntax
        const parser = new xml2js.Parser({ explicitArray: false });
        const parseString = util.promisify(parser.parseString);
        let destructiveChanges = await parseString(fs.readFileSync(path.resolve(destructiveManifestFile)));
        if (kit_1.isEmpty(destructiveChanges["Package"]["types"])) {
            throw new core_1.SfdxError("Invalid Destructive Change Definitiion Encountered");
        }
        sfpowerkit_1.SFPowerkit.log(destructiveChanges["Package"]["types"], sfpowerkit_1.LoggerLevel.TRACE);
    }
    generateEmptyPackageXml(workingDirectory, apiversion) {
        let packageXml = `<?xml version="1.0" encoding="UTF-8"?>
    <Package xmlns="http://soap.sforce.com/2006/04/metadata">
        <types>
            <members>*</members>
            <name>CustomLabel</name>
        </types>
        <version>${apiversion}</version>
    </Package>`;
        let packageXmlPath = path.join(workingDirectory, "package.xml");
        fs.outputFileSync(packageXmlPath, packageXml);
        sfpowerkit_1.SFPowerkit.log(`Empty Package.xml with ${apiversion} created at ${workingDirectory}`, sfpowerkit_1.LoggerLevel.DEBUG);
    }
    async generateDeploymentZipFile(workingDirectory) {
        let zipFile = path.join(fileutils_1.default.getGlobalCacheDir(), "package.zip");
        await zipDirectory_1.zipDirectory(workingDirectory, zipFile);
        return zipFile;
    }
    async deployDestructiveManifest(zipFile, conn) {
        //Deploy Package
        conn.metadata.pollTimeout = 300;
        let deployId;
        var zipStream = fs.createReadStream(zipFile);
        await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
            if (error) {
                sfpowerkit_1.SFPowerkit.log(error.message, sfpowerkit_1.LoggerLevel.ERROR);
            }
            deployId = result;
        });
        sfpowerkit_1.SFPowerkit.log(`Deploying Destructive Changes with ID ${deployId.id} to ${this.org.getUsername()}`, sfpowerkit_1.LoggerLevel.INFO);
        let metadata_deploy_result = await checkDeploymentStatus_1.checkDeploymentStatus(conn, deployId.id);
        if (metadata_deploy_result.success) {
            if (metadata_deploy_result.success)
                sfpowerkit_1.SFPowerkit.log(`Deployed Destructive Changes  in target org ${this.org.getUsername()} succesfully`, sfpowerkit_1.LoggerLevel.INFO);
        }
        else {
            throw new core_1.SfdxError(`Unable to deploy the Destructive Changes: ${metadata_deploy_result.details["componentFailures"]["problem"]}`);
        }
    }
}
exports.default = Destruct;
Destruct.description = messages.getMessage("commandDescription");
Destruct.examples = [
    `$ sfdx sfpowerkit:org:destruct -m destructiveChanges.xml -u prod@prod3.com`
];
Destruct.flagsConfig = {
    manifest: command_1.flags.filepath({
        required: false,
        char: "m",
        description: messages.getMessage("destructiveManifestFlagDescription")
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL"
        ]
    })
};
// Set this to true if your command requires a project workspace; 'requiresProject' is false by default
// protected static requiresProject = true;
// Comment this out if your command does not require an org username
Destruct.requiresUsername = true;
//# sourceMappingURL=destruct.js.map