"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const ts_types_1 = require("@salesforce/ts-types");
const fs_extra_1 = __importDefault(require("fs-extra"));
const command_1 = require("@salesforce/command");
const rimraf = require("rimraf");
const core_1 = require("@salesforce/core");
const xml2js = require("xml2js");
const util = require("util");
// tslint:disable-next-line:ordered-imports
var jsforce = require("jsforce");
var path = require("path");
const checkRetrievalStatus_1 = require("../../../../utils/checkRetrievalStatus");
const checkDeploymentStatus_1 = require("../../../../utils/checkDeploymentStatus");
const extract_1 = require("../../../../utils/extract");
const zipDirectory_1 = require("../../../../utils/zipDirectory");
const sfpowerkit_1 = require("../../../../sfpowerkit");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "matchingrule_deactivate");
class Deactivate extends command_1.SfdxCommand {
    async run() {
        rimraf.sync("temp_sfpowerkit");
        sfpowerkit_1.SFPowerkit.setLogLevel(this.flags.loglevel, this.flags.json);
        //Connect to the org
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        const apiversion = await conn.retrieveMaxApiVersion();
        let retrieveRequest = {
            apiVersion: apiversion
        };
        //Retrieve Duplicate Rule
        retrieveRequest["singlePackage"] = true;
        retrieveRequest["unpackaged"] = {
            types: { name: "MatchingRules", members: this.flags.name }
        };
        conn.metadata.pollTimeout = 60;
        let retrievedId;
        await conn.metadata.retrieve(retrieveRequest, function (error, result) {
            if (error) {
                return console.error(error);
            }
            retrievedId = result.id;
        });
        let metadata_retrieve_result = await checkRetrievalStatus_1.checkRetrievalStatus(conn, retrievedId, !this.flags.json);
        if (!metadata_retrieve_result.zipFile)
            throw new core_1.SfdxError("Unable to find the requested Matching Rule");
        //Extract Matching Rule
        var zipFileName = "temp_sfpowerkit/unpackaged.zip";
        fs_extra_1.default.mkdirSync("temp_sfpowerkit");
        fs_extra_1.default.writeFileSync(zipFileName, metadata_retrieve_result.zipFile, {
            encoding: "base64"
        });
        await extract_1.extract(`./temp_sfpowerkit/unpackaged.zip`, "temp_sfpowerkit");
        fs_extra_1.default.unlinkSync(zipFileName);
        let resultFile = `temp_sfpowerkit/matchingRules/${this.flags.name}.matchingRule`;
        if (fs_extra_1.default.existsSync(path.resolve(resultFile))) {
            const parser = new xml2js.Parser({ explicitArray: false });
            const parseString = util.promisify(parser.parseString);
            let retrieve_matchingRule = await parseString(fs_extra_1.default.readFileSync(path.resolve(resultFile)));
            this.ux.log(`Retrieved Matching Rule  for Object : ${this.flags.name}`);
            if (ts_types_1.isJsonArray(retrieve_matchingRule.MatchingRules.matchingRules)) {
                retrieve_matchingRule.MatchingRules.matchingRules.forEach(element => {
                    element.ruleStatus = "Inactive";
                });
            }
            else {
                if (!util.isNullOrUndefined(retrieve_matchingRule.MatchingRules.matchingRules))
                    retrieve_matchingRule.MatchingRules.matchingRules.ruleStatus =
                        "Inactive";
                else {
                    throw new core_1.SfdxError("No Custom Matching Rule  found in the org");
                }
            }
            //Deactivate Rule
            this.ux.log(`Preparing Deactivation`);
            let builder = new xml2js.Builder();
            var xml = builder.buildObject(retrieve_matchingRule);
            fs_extra_1.default.writeFileSync(resultFile, xml);
            var zipFile = "temp_sfpowerkit/package.zip";
            await zipDirectory_1.zipDirectory("temp_sfpowerkit", zipFile);
            //Deploy Rule
            conn.metadata.pollTimeout = 300;
            let deployId;
            var zipStream = fs_extra_1.default.createReadStream(zipFile);
            await conn.metadata.deploy(zipStream, { rollbackOnError: true, singlePackage: true }, function (error, result) {
                if (error) {
                    return console.error(error);
                }
                deployId = result;
            });
            this.ux.log(`Deploying Deactivated Matching Rule with ID  ${deployId.id}  to ${this.org.getUsername()}`);
            let metadata_deploy_result = await checkDeploymentStatus_1.checkDeploymentStatus(conn, deployId.id);
            if (!metadata_deploy_result.success)
                throw new core_1.SfdxError(`Unable to deploy the deactivated matching rule : ${metadata_deploy_result.details["componentFailures"]["problem"]}`);
            this.ux.log(`Matching Rule for ${this.flags.name} deactivated`);
            return 0;
        }
        else {
            throw new core_1.SfdxError("Matching Rule not found in the org");
        }
    }
}
exports.default = Deactivate;
Deactivate.description = messages.getMessage("commandDescription");
Deactivate.examples = [
    `$ sfdx sfpowerkit:org:matchingrules:deactivate -n Account -u sandbox
    Polling for Retrieval Status
    Retrieved Matching Rule  for Object : Account
    Preparing Deactivation
    Deploying Deactivated Rule with ID  0Af4Y000003OdTWSA0
    Polling for Deployment Status
    Polling for Deployment Status
    Matching Rule for  Account deactivated
  `
];
Deactivate.flagsConfig = {
    name: command_1.flags.string({
        required: true,
        char: "n",
        description: messages.getMessage("nameFlagDescription")
    }),
    loglevel: command_1.flags.enum({
        description: "logging level for this command invocation",
        default: "info",
        required: false,
        options: [
            "trace",
            "debug",
            "info",
            "warn",
            "error",
            "fatal",
            "TRACE",
            "DEBUG",
            "INFO",
            "WARN",
            "ERROR",
            "FATAL"
        ]
    })
};
// Comment this out if your command does not require an org username
Deactivate.requiresUsername = true;
//# sourceMappingURL=deactivate.js.map