"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const command_1 = require("@salesforce/command");
const core_1 = require("@salesforce/core");
const fs = require("fs-extra");
const path = __importStar(require("path"));
const fileutils_1 = __importDefault(require("../../../utils/fileutils"));
const request = require("request-promise-native");
const rimraf = require("rimraf");
const querystring = require("querystring");
// Initialize Messages with the current plugin directory
command_1.core.Messages.importMessagesDirectory(__dirname);
// Load the specific messages for this file. Messages from @salesforce/command, @salesforce/core,
// or any library that is using the messages framework can also be loaded this way.
const messages = command_1.core.Messages.loadMessages("sfpowerkit", "org_coverage");
class OrgCoverage extends command_1.SfdxCommand {
    async run() {
        rimraf.sync("temp_sfpowerkit");
        if (this.flags.output && !this.flags.format) {
            throw new core_1.SfdxError("format is required to generate the output");
        }
        else if (this.flags.format && !this.flags.output) {
            throw new core_1.SfdxError("output path is required to generate the output");
        }
        await this.org.refreshAuth();
        const conn = this.org.getConnection();
        this.flags.apiversion =
            this.flags.apiversion || (await conn.retrieveMaxApiVersion());
        var apexcoverage = new ApexCoverage();
        apexcoverage.coverage = await this.getApexCoverage(conn);
        if (this.flags.outputfile) {
            await fs.outputJSON(this.flags.outputfile, apexcoverage);
        }
        this.ux.log(`Successfully Retrieved the Apex Test Coverage of the org ${this.org.getOrgId()} `);
        this.ux.log(`coverage:${apexcoverage.coverage}`);
        const classCoverage = await this.getApexCoverageByDetails(conn, this.flags.output);
        return { coverage: apexcoverage.coverage, classCoverage: classCoverage };
    }
    async getApexCoverage(conn) {
        var encoded_querystring = querystring.escape(`SELECT PercentCovered FROM ApexOrgWideCoverage`);
        var query_uri = `${conn.instanceUrl}/services/data/v${this.flags.apiversion}/tooling/query?q=${encoded_querystring}`;
        const coverage_score_query_result = await request({
            method: "get",
            url: query_uri,
            headers: {
                Authorization: `Bearer ${conn.accessToken}`
            },
            json: true
        });
        // this.ux.logJson(health_score_query_result);
        return coverage_score_query_result.records[0].PercentCovered;
    }
    async getApexCoverageByDetails(conn, outputDir) {
        let query = "SELECT ApexClassOrTriggerId, ApexClassOrTrigger.Name, NumLinesCovered, NumLinesUncovered, coverage FROM ApexCodeCoverageAggregate ORDER BY ApexClassOrTrigger.Name";
        const results = (await conn.tooling.query(query));
        const output = [];
        if (results.size > 0) {
            results.records.forEach(element => {
                let percentage = element.NumLinesCovered === 0
                    ? 0
                    : Math.round((element.NumLinesCovered /
                        (element.NumLinesCovered + element.NumLinesUncovered)) *
                        100);
                output.push({
                    id: element.ApexClassOrTriggerId,
                    name: element.ApexClassOrTrigger.Name,
                    type: element.ApexClassOrTrigger.attributes.url.split("/")[6],
                    percentage: `${percentage}%`,
                    comments: percentage < 75
                        ? "Action required"
                        : percentage < 85 && percentage >= 75
                            ? "Looks fine but target more than 85%"
                            : "",
                    uncoveredLines: element.Coverage.uncoveredLines.join(";")
                });
            });
            this.ux.table(output, [
                "id",
                "name",
                "type",
                "percentage",
                "comments",
                "uncoveredLines"
            ]);
            if (this.flags.format && this.flags.format === "json") {
                rimraf.sync(outputDir);
                await this.generateJsonOutput(output, outputDir);
            }
            else if (this.flags.format && this.flags.format === "csv") {
                rimraf.sync(outputDir);
                await this.generateCSVOutput(output, outputDir);
            }
        }
        return output;
    }
    async generateJsonOutput(testResult, outputDir) {
        let outputJsonPath = `${outputDir}/output.json`;
        let dir = path.parse(outputJsonPath).dir;
        if (!fs.existsSync(dir)) {
            fileutils_1.default.mkDirByPathSync(dir);
        }
        fs.writeFileSync(outputJsonPath, JSON.stringify(testResult));
        this.ux.log(`Output ${outputDir}/output.json is generated successfully`);
    }
    async generateCSVOutput(testResult, outputDir) {
        let outputcsvPath = `${outputDir}/output.csv`;
        let dir = path.parse(outputcsvPath).dir;
        if (!fs.existsSync(dir)) {
            fileutils_1.default.mkDirByPathSync(dir);
        }
        let newLine = "\r\n";
        let output = "ID,NAME,TYPE,PERCENTAGE,COMMENTS,UNCOVERED LINES" + newLine;
        testResult.forEach(element => {
            output = `${output}${element.id},${element.name},${element.type},${element.percentage},${element.comments},${element.uncoveredLines}${newLine}`;
        });
        fs.writeFileSync(outputcsvPath, output);
        this.ux.log(`Output ${outputDir}/output.csv is generated successfully`);
    }
}
exports.default = OrgCoverage;
OrgCoverage.description = messages.getMessage("commandDescription");
OrgCoverage.examples = [
    `$  sfdx sfpowerkit:org:orgcoverage  -u myOrg@example.com
  sfdx sfpowerkit:org:orgcoverage  -u myOrg@example.com -d testResult -f csv
  sfdx sfpowerkit:org:orgcoverage  -u myOrg@example.com -d testResult -f json


  Successfully Retrieved the Apex Test Coverage of the org XXXX
  coverage:85
  ID     		NAME                  TYPE          PERCENTAGE    COMMENTS                              UNCOVERED LINES
  ───────		──────────────────    ────────      ──────────	  ───────────────────────────────────   ──────────────────
  01pxxxx		sampleController      ApexClass     100%
  01pxxxx		sampletriggerHandler  ApexClass	    80%           Looks fine but target more than 85%   62;76;77;
  01pxxxx		sampleHelper          ApexClass	    72%           Action required                       62;76;77;78;98;130;131
  01qxxxx		sampleTrigger         ApexTrigger   100%
  Output testResult/output.csv is generated successfully
  `
];
OrgCoverage.flagsConfig = {
    output: command_1.flags.string({
        char: "d",
        description: messages.getMessage("outputFolderDescription"),
        required: false
    }),
    format: command_1.flags.enum({
        required: false,
        char: "f",
        description: messages.getMessage("formatFlagDescription"),
        options: ["json", "csv"]
    })
};
// Comment this out if your command does not require an org username
OrgCoverage.requiresUsername = true;
class ApexCoverage {
}
exports.ApexCoverage = ApexCoverage;
//# sourceMappingURL=orgcoverage.js.map